@@ -1,11 +1,11 @@
-use proc_macro2::Span;
+use proc_macro2::{Span, TokenStream};
 use quote::{quote, ToTokens};
 use syn::spanned::Spanned;
-use syn::{GenericArgument, Ident, Path, PathArguments, PathSegment, TraitItemType, Type};
+use syn::{GenericArgument, Ident, PathArguments, PathSegment, TraitItemType, Type};
 
+use crate::match_assoc_type;
 use crate::parse_trait_sig::TypeTransform;
-use crate::syn_utils::{find_in_type, lifetime_bounds, trait_bounds};
-use crate::AssocTypeMatcher;
+use crate::syn_utils::{iter_type, lifetime_bounds, trait_bounds};
 
 #[derive(Debug)]
 pub enum AssocTypeParseError {
@@ -16,16 +16,16 @@ pub enum AssocTypeParseError {
 
 #[derive(Debug, Clone)]
 pub struct BoxType {
-    trait_name: Path,
-    placeholder_lifetime: bool,
+    pub inner: TokenStream,
+    pub placeholder_lifetime: bool,
 }
 
 impl ToTokens for BoxType {
     fn to_tokens(&self, tokens: &mut proc_macro2::TokenStream) {
-        let path = &self.trait_name;
+        let inner = &self.inner;
         match self.placeholder_lifetime {
-            true => tokens.extend(quote! {Box<dyn #path + '_>}),
-            false => tokens.extend(quote! {Box<dyn #path>}),
+            true => tokens.extend(quote! {Box<#inner + '_>}),
+            false => tokens.extend(quote! {Box<#inner>}),
         }
     }
 }
@@ -63,7 +63,7 @@ pub fn parse_assoc_type(
             if ident == "Into" && args.args.len() == 1 {
                 if let GenericArgument::Type(into_type) = args.args.first().unwrap() {
                     // provide a better error message for type A: Into<Self::B>
-                    if find_in_type(into_type, &AssocTypeMatcher).is_some() {
+                    if iter_type(into_type).any(match_assoc_type) {
                         return Err((into_type.span(), AssocTypeParseError::AssocTypeInBound));
                     }
 
@@ -79,10 +79,11 @@ pub fn parse_assoc_type(
                 }
             }
         }
+        let path = &bound.path;
         return Ok((
             &assoc_type.ident,
             DestType::Box(BoxType {
-                trait_name: bound.path.clone(),
+                inner: quote! {dyn #path},
                 placeholder_lifetime: !lifetime_bounds(&assoc_type.bounds)
                     .any(|l| l.ident == "static"),
             }),
