@@ -76,6 +76,16 @@ trait Foo<X> {
     fn foobar(&self, x: X) -> Self::A;
 }
 
+#[dynamize::dynamize]
+trait FooWithWhere<X>
+where
+    X: std::fmt::Display,
+{
+    type A: Into<String>;
+
+    fn foobar(&self, x: X) -> Self::A;
+}
+
 #[dynamize::dynamize]
 trait Bar<X> {
     fn foobar<A>(&self, x: X) -> A;
@@ -136,6 +146,10 @@ trait TraitWithCallback {
     type A: Into<String>;
     fn fun_with_callback<F: Fn(Self::A)>(&self, a: F);
 
+    fn fun_with_callback0<G>(&self, a: G)
+    where
+        G: Fn(Self::A);
+
     fn fun_with_callback1<X: Fn(Option<Self::A>)>(&self, a: X);
 
     fn fun_with_callback2<Y: Fn(i32, Option<Self::A>, String) -> bool>(&self, a: Y);
@@ -177,3 +191,33 @@ fn type_trait_bound_static_test(
 ) -> Option<Box<dyn std::error::Error>> {
     t.foobar("test").err()
 }
+
+#[dynamize::dynamize]
+trait TypeWithSuper: std::fmt::Display + Sized {
+    type A: std::error::Error;
+
+    fn foobar(&self, text: &str) -> Result<i32, Self::A>;
+}
+
+fn test3<T: TypeWithSuper>(some: T) {
+    let dyn_trait: &dyn DynTypeWithSuper = &some;
+    println!("{}", dyn_trait);
+}
+
+#[dynamize::dynamize]
+trait ReturnIter {
+    type A: std::error::Error;
+
+    fn foobar(&mut self, text: &str) -> &mut dyn Iterator<Item = Self::A>;
+}
+
+#[dynamize::dynamize]
+trait FunIter {
+    type A: std::error::Error;
+
+    fn foobar<F: Fn(&mut dyn Iterator<Item = Self::A>)>(&mut self, f: F);
+
+    fn foobar1<G: Fn(Option<&mut dyn Iterator<Item = Self::A>>)>(&mut self, f: G);
+
+    fn foobar2<H: Fn(&mut dyn Iterator<Item = &mut dyn Iterator<Item = Self::A>>)>(&mut self, f: H);
+}
